/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package biteme;

/**
 * Represents the status bitflags of a single Tile.
 * @author NFW Thissen
 * @version 0.0.1
 */
public class Status {

    /** Represents a Tile that does not contain anything. */
    public static final int NONE = 0;
    /** Represents a Tile that is a wall. */
    public static final int WALL = 1;
    /** Represents a Tile that contains a coin. */
    public static final int COIN = 2;
    /** Represents a Tile that contains a box. */
    public static final int BOX = 4;
    /** Represents a Tile that is a one-way up Tile. */
    public static final int ONE_WAY_UP = 8;
    /** Represents a Tile that is a one-way down Tile. */
    public static final int ONE_WAY_DOWN = 16;
    /** Represents a Tile that is a one-way left Tile. */
    public static final int ONE_WAY_LEFT = 32;
    /** Represents a Tile that is a one-way right Tile. */
    public static final int ONE_WAY_RIGHT = 64;
    private int value;

    /**
     * Constructor.
     */
    public Status() {
        this.value = 0;
    }

    /**
     * Gets the integer status value. Only used for saving a level.
     * @return The integer value of the status.
     */
    public int getValue() {
        return this.value;
    }

    /**
     * Sets the integer status value. Only used for loading a level.
     * @param val The integer value of the status.
     */
    public void setValue(int val) {
        this.value = val;
    }

    /**
     * Sets a bit in the status integer.
     * @param val Integer value determining which bit to set.
     */
    private void set(int val) {
        this.value |= val;
    }

    /**
     * Clears a bit in the status integer.
     * @param val Integer value determining which bit to clear.
     */
    private void clear(int val) {
        this.value &= ~val;
    }

    /**
     * Toggles a bit in the status integer.
     * @param val Integer value determining which bit to toggle.
     */
    private void toggle(int val) {
        this.value ^= val;
    }

    /** Adds a wall to the current status. */
    public void setWall() {
        this.set(Status.WALL);
    }

    /** Removes a wall from the current status. */
    public void clearWall() {
        this.clear(Status.WALL);
    }

    /** Toggles the wall status. */
    public void toggleWall() {
        this.toggle(Status.WALL);
    }

    /** Adds a coin to the current status. */
    public void setCoin() {
        this.set(Status.COIN);
    }

    /** Removes a coin from the current status. */
    public void clearCoin() {
        this.clear(Status.COIN);
    }

    /** Toggles the coin status. */
    public void toggleCoin() {
        this.toggle(Status.COIN);
    }

    /** Adds a box to the current status. */
    public void setBox() {
        this.set(Status.BOX);
    }

    /** Removes a box from the current status. */
    public void clearBox() {
        this.clear(Status.BOX);
    }

    /** Toggles the box status. */
    public void toggleBox() {
        this.toggle(Status.BOX);
    }

    /**
     * Adds the one-way tile status.
     * @param d The direction of the one-way tile.
     */
    public void setOneWay(Direction d) {
        switch (d) {
            case UP:
                this.set(Status.ONE_WAY_UP);
                break;
            case DOWN:
                this.set(Status.ONE_WAY_DOWN);
                break;
            case LEFT:
                this.set(Status.ONE_WAY_LEFT);
                break;
            case RIGHT:
                this.set(Status.ONE_WAY_RIGHT);
                break;
        }
    }

    /**
     * Removes the one-way tile status.
     * @param d The direction of the one-way tile.
     */
    public void clearOneWay(Direction d) {
        switch (d) {
            case UP:
                this.clear(Status.ONE_WAY_UP);
                break;
            case DOWN:
                this.clear(Status.ONE_WAY_DOWN);
                break;
            case LEFT:
                this.clear(Status.ONE_WAY_LEFT);
                break;
            case RIGHT:
                this.clear(Status.ONE_WAY_RIGHT);
                break;
        }
    }

    /** Toggles the OneWay status. */
    public void toggleOneWay(Direction d) {
        switch (d) {
            case UP:
                this.toggle(Status.ONE_WAY_UP);
                break;
            case DOWN:
                this.toggle(Status.ONE_WAY_DOWN);
                break;
            case LEFT:
                this.toggle(Status.ONE_WAY_LEFT);
                break;
            case RIGHT:
                this.toggle(Status.ONE_WAY_RIGHT);
                break;
        }
    }

    /**
     * Checks if the current status contains the specified value.
     * @param val The value to check for.
     * @return True if the status contains the specified value.
     */
    private boolean contains(int val) {
        return (this.value & val) == val;
    }

    /**
     * Checks if the current status contains nothing (no wall, coin, box, or OneWayTile).
     * @return Whether the current status contains nothing.
     */
    public boolean containsNothing() {
        return (value == 0);
    }

    /**
     * Checks if the current status contains a wall.
     * @return True if the current status contains a wall.
     */
    public boolean containsWall() {
        return this.contains(WALL);
    }

    /**
     * Checks if the current status contains a coin.
     * @return True if the current status contains a coin.
     */
    public boolean containsCoin() {
        return this.contains(COIN);
    }

    /**
     * Checks if the current status contains a box.
     * @return True if the current status contains a box.
     */
    public boolean containsBox() {
        return this.contains(BOX);
    }

    /**
     * Checks if the current status contains a one-way up tile.
     * @return True if the current status contains a one-way up tile.
     */
    public boolean containsOneWayUp() {
        return this.contains(ONE_WAY_UP);
    }

    /**
     * Checks if the current status contains a one-way down tile.
     * @return True if the current status contains a one-way down tile.
     */
    public boolean containsOneWayDown() {
        return this.contains(ONE_WAY_DOWN);
    }

    /**
     * Checks if the current status contains a one-way left tile.
     * @return True if the current status contains a one-way left tile.
     */
    public boolean containsOneWayLeft() {
        return this.contains(ONE_WAY_LEFT);
    }

    /**
     * Checks if the current status contains a one-way right tile.
     * @return True if the current status contains a one-way right tile.
     */
    public boolean containsOneWayRight() {
        return this.contains(ONE_WAY_RIGHT);
    }

    /**
     * Checks whether the current status contains any one-way Tile.
     * @return True if the current status contains any one-way Tile.
     */
    public boolean containsOneWayTile() {
        return this.containsOneWayUp() || this.containsOneWayDown() || this.containsOneWayLeft() || this.containsOneWayRight();
    }
}
